/***************************************************************************
 *   $Id: checking2html.c,v 1.5 2007/02/20 10:11:40 kenjen Exp $
 *                                                                         *
 *   Created: Mon Sep 30 2002
 *                                                                         *
 *   Copyright (C) 2002-2005 by Kenneth Jennings                           *
 *   ken_jennings@bellsouth.net                                            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "miscUtils.h"
#include "checking2html.h"

static char rcsid[] = "$Id: checking2html.c,v 1.5 2007/02/20 10:11:40 kenjen Exp $";

char * getRCSIDchecking2html( void ) { return rcsid; }

static char zs_FileData[4097]; /* Line of text read from the data file. */


static CheckingEntry * CheckingList = NULL;

/* List of fields for the Checking file */

static CsvData zz_Date     = { CSV_INIT, CSV_DATE1,                    3, 10 }; /* M/D = 1/1 or 12/31 */
static CsvData zz_Desc     = { CSV_INIT, CSV_NULL    | CSV_EVERYTHING, 1, 75 }; /* Free text describing entry */
static CsvData zz_Withdraw = { CSV_INIT, CSV_NULL    | CSV_MONEY2,     0, 11 }; /* Checks/Withdrawals (-) */
static CsvData zz_Deposit  = { CSV_INIT, CSV_NULL    | CSV_MONEY2,     0, 11 }; /* Contributions/Deposits (+) */
static CsvData zz_Balance  = { CSV_INIT, CSV_MINSIZE | CSV_MONEY2,     0, 11 }; /* Balance */

static CsvData * inputFields[] =
{
	&zz_Date,
	&zz_Desc,
	&zz_Withdraw,
	&zz_Deposit,
	&zz_Balance,
	NULL
};


Checking * newChecking(void)
{
	Checking * p;

	p = (Checking *)malloc(sizeof(Checking));

	if (!p)
		return NULL;

	p->date        = NULL;
	p->description = NULL;
	p->withdrawals = NULL;
	p->deposits    = NULL;
	p->balance     = NULL;

	return p;
}



CheckingEntry * newCheckingEntry(void)
{
	CheckingEntry * p;

	p = (CheckingEntry *)malloc(sizeof(CheckingEntry));

	if (!p)
		return NULL;

	p->next = NULL;
	p->txn  = NULL;

	return p;
}


bool destroyChecking(Checking * p)
{
	if (!p)
		return false;

	if (p->date)
		free(p->date);

	if (p->description)
		free(p->description);

	if (p->withdrawals)
		free(p->withdrawals);

	if (p->deposits)
		free(p->deposits);

	if (p->balance)
		free(p->balance);

	free(p);
	return true;
}


bool destroyCheckingEntry(CheckingEntry * p)
{
	if (!p)
		return false;

	free(p);
	return true;
}


void debugChecking(void)
{
	CheckingEntry * p = CheckingList;

	if (!p)
		return;

	while (p)
	{
		if (p->txn)
			fprintf(stderr,
					"TXN: DT(%s) DESC(%s) WD(%s) DP(%s) BAL(%s)\n",
					p->txn->date        ? p->txn->date        : "NULL",
					p->txn->description ? p->txn->description : "NULL",
					p->txn->withdrawals ? p->txn->withdrawals : "NULL",
					p->txn->deposits    ? p->txn->deposits    : "NULL",
					p->txn->balance     ? p->txn->balance     : "NULL" );
		else
			fprintf(stderr,
					"TXN: ???\n");

		p = p->next;
	}
}


CheckingEntry * addChecking(Checking * p)
{
	CheckingEntry * prevP = NULL;
	CheckingEntry * thisP = CheckingList;
	CheckingEntry * newP;


	if (!p || !p->date)
		return NULL;

	newP = newCheckingEntry();

	if (!newP)
		return NULL;

	newP->txn  = p;
	newP->next = NULL;

	if (!CheckingList)
	{
		CheckingList = newP;
		return newP;
	}

	while (thisP)
	{
		prevP = thisP;
		thisP = thisP->next;
	}

	prevP->next = newP;

	return newP;
}


Checking * strtoChecking(char * date,
						 char * desc,
						 char * withdraw,
						 char * deposit,
						 char * balance)
{
	char       money[20];
	Checking * p;

	if (!date    || !*date   ||
		!balance || !*balance)
		return NULL;

	p = newChecking();

	if (!p)
		return NULL;

	p->date        = strdup(date);
	p->description = strdup(desc);

	if (!moneyStr(withdraw, money) )
		p->withdrawals = strdup(money);
	else
		p->withdrawals = strdup(withdraw);

	if (!moneyStr(deposit, money) )
		p->deposits = strdup(money);
	else
		p->deposits = strdup(deposit);

	if (!moneyStr(balance, money) )
		p->balance = strdup(money);
	else
		p->balance = strdup(balance);

	return p;
}


/* Travel the list, add up how many entries are in a given month */

int countMonthTxns(int month)
{
	CheckingEntry * p     = CheckingList;
	int             mon;
	int             count = 0;


	if ( !p || (month < 1) || (month > 12) )
		return 0;

	while(p)
	{
		if (p->txn)
		{
			if (p->txn->date)
			{
				mon = atoi(p->txn->date);

				if (mon == month)
					count++;
			}
		}

		p = p->next;
	}

	return count;
}

/* Add new Checking Transaction to Checking list in order of date (as they are read) */
/* Simply create a new Checking and add it to the list.
 * fields[0] = zz_Date;
 * fields[1] = zz_Desc;
 * fields[2] = zz_Withdraw;
 * fields[3] = zz_Deposit;
 * fields[4] = zz_Balance;
 */

bool processChecking(CsvData ** fields)
{
	Checking * p;
	char     * s;
	int        this_day;
	int        this_month;

	if (!fields || !*fields)
		return false;

	/* Do some simple data validation first...
	 * Date and Balance must have data.  Also, Date must be somehow usable M/D 1/1 to 12/31.
	 * Either Withdraw or Deposit must have data, but not both.
	 * Desc may be blank.
	 */

	if ( (!fields[0]->buf || !fields[0]->buf[0])     || /* Date    */
		 (!fields[4]->buf || !fields[4]->buf[0])     || /* Total   */
		 ( (fields[2]->buf && fields[2]->buf[0])     && /* Withdraw filled AND*/
		   (fields[3]->buf && fields[3]->buf[0]) ) )    /* Deposit filled? */
		 return false;

	this_month = atoi(fields[0]->buf);
	s = strchr(fields[0]->buf, '/');

	if ((this_month < 1) || (this_month > 12) || !s)
		return false;

	s++; /* skip over the slash */
	this_day = atoi(s);

	if ((this_day < 1) || (this_day > 31)) /* Not real date checking */
		return false;

	/* If the year follows, remove it */
	s = strchr(s, '/');
	if (s)
		*s = '\0';

	p = strtoChecking(fields[0]->buf,   /* Date */
					  fields[1]->buf,   /* Desc */
					  fields[2]->buf,   /* Withdraw */
					  fields[3]->buf,   /* Deposit */
					  fields[4]->buf ); /* Balance */

	if (!p)                /* Failed to make a new Checking? */
		return false;

	addChecking(p); /* Checking Entry associates Checking to Date (more or less)*/

	return true;
}


CheckingEntry * gimmeCheckingList(void)
{
	return CheckingList;
}


/*
 * Read a line from the file and remove the EOL
 */

static char * readAndUnEOL(FILE * fp)
{
	char * inString;
	int    index = 0;
	int    len;


	if (!fp)
		return NULL;

	inString = fgets(zs_FileData,
					 sizeof(zs_FileData),
					 fp);

	if (!inString) /* Reached the end of the file! */
		return NULL;

	zs_FileData[sizeof(zs_FileData) - 1] = '\0'; /* Just to make sure it ends. */

	len = strlen(zs_FileData);

	if (!len) /* Don't even bother doing the rest of the code on an empty string */
		return inString;


	/* Nuke the end of line...   */
	/* Yeah, array subscripts are a whole lot less efficient than     */
	/* pointers, but this needs to work as stably as possible first.  */

	while ( (index < len) &&    /* Loop through the length of the string */
			zs_FileData[index]) /* and while there's no null terminator. */
	{
		if ((zs_FileData[index] == 0x0a) || /* (LF == ascii 10) */
			(zs_FileData[index] == 0x0d))   /* (CR == ascii 13) */
		{
			zs_FileData[index] = '\0';
			continue;
		}

		index++;
	}

	return inString;
}



bool LoadCSVFile(FILE     * fp,
				 CsvData ** fieldSpecs,
				 bool    (* loadFunc)(CsvData ** ) )
{
	int    lineCount = 0;
	char * inString  = zs_FileData; /* To get into the while() the first time */
	int    retVal;

	if (!fp || !fieldSpecs || !loadFunc)
	{
		fprintf(stderr,
				"LoadCSVFile(): Invalid (NULL) parameter\n");
		return false;
	}

	while (inString != NULL) /* Until we run out of file to read */
	{
		inString = readAndUnEOL(fp);

		if (!inString) /* Reached the end of the file! */
			continue;

		lineCount++;

		/* Is this in the right format to process? */
		retVal = csvParse(inString,
						  fieldSpecs, /* See declarations of CsvData at start of program */
						  ',');
	
		if (retVal != CSV_OK)
		{
			fprintf(stderr,
					"CSVerr: %d, Skipping Line %d: %s\n",
					retVal,
					lineCount,
					inString);
			continue;
		}

		if (! (* loadFunc)(fieldSpecs) )
		{
			fprintf(stderr,
					"loadFunc(*) did not process data on Line %d: %s\n",
					lineCount,
					inString);
			continue;
		}
	}

	csvCleanUp(fieldSpecs);

	return true;
}


bool LoadChecking(FILE * inputFile)
{
	if (!LoadCSVFile(inputFile,
					 inputFields,
					 processChecking) )
	{
		fprintf(stderr,
				"LoadChecking() received failure from LoadCSVFile()\n");
		return false;
	}

	return true;
}

/* Initiate Table.
 * do the Header row.
 * Loop through each checking transaction.
 *   Output Row Header for this row in green or grey.
 *   If this is a new month,
 *     determine how many transactions belong in this month.
 *     the first cell is the vertical columnspan for the name of the month.
 *   output the transaction cell data.
 * End loop.
 * end table.
 */

bool buildHTML(FILE          * out,
			   CheckingEntry * p,
			   char          * ltext)
{
	int    last_month  = 0; /* preps table output */
	int    this_month  = 1;
	int    this_day    = 0;
	char * s;
	int    count_month = 0;
	bool   flipcolorD  = false; /* Date cells: false = dark 8888DD. true = light AAAAFF */
	bool   flipcolorR  = false; /* Each Row: false = grey DDDDDD. true = green AAFFAA */

	/* A list of month names for the vertical labels */

	char * monthNames[] =
	{
		"J<br>A<br>N",
		"F<br>E<br>B",
		"M<br>A<br>R",
		"A<br>P<br>R",
		"M<br>A<br>Y",
		"J<br>U<br>N",
		"J<br>U<br>L",
		"A<br>U<br>G",
		"S<br>E<br>P",
		"O<br>C<br>T",
		"N<br>O<br>V",
		"D<br>E<br>C",
		NULL
	};

	char * endings[] =
	{
		/*  1 */ "st",
		/*  2 */ "nd",
		/*  3 */ "rd",
		/*  4 */ "th",
		/*  5 */ "th",
		/*  6 */ "th",
		/*  7 */ "th",
		/*  8 */ "th",
		/*  9 */ "th",
		/* 10 */ "th",
		/* 11 */ "th",
		/* 12 */ "th",
		/* 13 */ "th",
		/* 14 */ "th",
		/* 15 */ "th",
		/* 16 */ "th",
		/* 17 */ "th",
		/* 18 */ "th",
		/* 19 */ "th",
		/* 20 */ "th",
		/* 21 */ "st",
		/* 22 */ "nd",
		/* 23 */ "rd",
		/* 24 */ "th",
		/* 25 */ "th",
		/* 26 */ "th",
		/* 27 */ "th",
		/* 28 */ "th",
		/* 29 */ "th",
		/* 30 */ "th",
		/* 31 */ "st",
		NULL
	};
	/* Start Table and set caption and headers */

	fprintf(out,
			"<table width=640 cellpadding=3 cellspacing=0 border=2>\n"
			"  <caption align=center valign=top>\n"
			"    <font size=5><b>%s</b></font></caption>\n" /* <- label text (ltext) goes here */
			"<tr bgcolor=#%s>\n"
			"  <th>&nbsp;</th>\n"
			"  <th>Date<br>&nbsp;Posted&nbsp;</th>\n"
			"  <th align=left>Description</th>\n"
			"  <th>Debits<br>(-)</th>\n"
			"  <th>Credits<br>(+)</th>\n"
			"  <th>Balance</th>\n"
			"</tr>\n",
			ltext,
			getHColor( HC_TITLE_ROW ) ); /* in the <caption> */


	while (p)
	{
		if (!p->txn)
		{
			p = p->next;
			continue;
		}

		/* When we loaded the txn data we validated the date data. */

		this_month = atoi(p->txn->date);
		s = strchr(p->txn->date, '/');
		s++;
		this_day = atoi(s);

		fprintf(out,
				"<tr bgcolor=#%s align=right>\n",
				flipcolorR ? getHColor( HC_EVEN_ROW ) : getHColor( HC_ODD_ROW ) ); /* Row color ? green : grey */

		flipcolorR = !flipcolorR;

		if (this_month != last_month)
		{
			count_month = countMonthTxns(this_month);
	
			fprintf(out,
					"  <td bgcolor=#%s rowspan=%d align=center>%s</td>\n",
					flipcolorD ? getHColor( HC_EVEN_MONTH ) : getHColor( HC_ODD_MONTH ), /* Month color ? lightblue : dkblue */
					count_month,
					monthNames[this_month - 1] );

			flipcolorD = !flipcolorD;
			last_month = this_month;
		}

		fprintf(out,
				"  <td align=center>%d<sup>%s</sup></td>\n"
				"  <td align=left>%s</td>\n"
				"  <td>%s</td>\n"
				"  <td>%s</td>\n"
				"  <td>%s</td>\n"
				"</tr>\n",
				this_day,
				endings[this_day - 1],
				nbspOrStr(p->txn->description),
				nbspOrStr(p->txn->withdrawals),
				nbspOrStr(p->txn->deposits),
				p->txn->balance );
				
		p = p->next;
	}

	fprintf(out,
			"</table>\n\n");

	return true;
}

