/***************************************************************************
 *   $Id: csvFile.c,v 1.5 2007/02/20 10:11:40 kenjen Exp $ *
 *                                                                         *
 *   Created: 2002/09/30 17:26:18                                          *
 *                                                                         *
 *   Copyright (C) 2002-2005 by Kenneth Jennings                           *
 *   ken_jennings@bellsouth.net                                            *
 *                                                                         *
 ***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <csvFile.h>

static char rcsid[] = "$Id: csvFile.c,v 1.5 2007/02/20 10:11:40 kenjen Exp $";

char * getRCSIDcsvFile( void ) { return rcsid; }

/*
 * Read a line from the file and remove the EOL
 */

static char * readAndUnEOL(FILE * fp)
{
	static char zz_buf[4097];
	char      * inString;
	char      * s     = zz_buf;
	int         index = 0;
	int         len;


	if (!fp)
		return NULL;

	inString = fgets(zz_buf, sizeof(zz_buf), fp);

	if (!inString) /* Reached the end of the file! */
		return NULL;

	zz_buf[sizeof(zz_buf) - 1] = '\0'; /* Just to make sure it ends. */

	len = strlen(zz_buf);

	if (!len) /* Don't even bother doing the rest of the code on an empty string */
		return inString;

	/* Nuke the end of line...   */

	while ( (index < len) &&    /* Loop through the length of the string */
			*s ) /* and while there's no null terminator. */
	{
		if ((*s == 0x0a) || /* (LF == ascii 10) */
			(*s == 0x0d) )  /* (CR == ascii 13) */
		{
			*s = '\0';
			continue;
		}

		index++;
		s++;
	}

	return inString;
}


static char * csvStrdup(char * s)
{
	int    n;
	char * c;

	if (!s)
		return NULL;

	n = strlen(s) + 1;

	c = (char *)malloc(n);

	if (c)
		strcpy(c, s);

	return c;
}


/* Given filled in columns (CsvData array), generate a row array
 * from the bufs.
 * Alloc an array of pointers for the right number of columns.
 * Fill in each pointer enty by doing a strdup of each buf.
 */

CsvRow * createCsvRow(CsvData ** colSpecs, int num)
{
	CsvRow  * row;
	char   ** buf;
	int       loop = 0;

	if (!colSpecs || !*colSpecs || (num < 1) )
		return NULL;

	/* We're being abusive here.  CsvRow is just a pointer
	 * to bufs.  bufs is a pointer to an array of pointers 
	 * to char strings.
	 */

	row = (CsvRow *)malloc(sizeof(CsvRow *));

	if (!row)
		return NULL;

	memset(row, 0, sizeof(CsvRow *));

	row->bufs = (char **)malloc(sizeof(char *) * num);
	
	if (!row->bufs)
	{
		free(row);
		return NULL;
	}
	
	memset(row->bufs, 0, sizeof(char *) * num);

	buf = row->bufs;

	while (loop < num)
	{
		if (*colSpecs)
			*buf = csvStrdup((*colSpecs)->buf);

		buf++;
		colSpecs++;
		loop++;
	}

	return row;
}


/* Nuke each element in the CsvData Array.  Then nuke the Array.
 * The NULL element is the end of the array.
 */

/*
freeCsvData(CsvData ** colSpecs)
{
	CsvData * colSpec;

	if (!colSpecs)
		return 0;

	colSpec = *colSpecs;

	while (colSpec)
	{
		free(colSpec);
		colSpec++;
	}

	free(colSpecs);

	return 0;
}
*/



/* Count all the usable nodes on the list
 */

long countCsvList(CsvList * list)
{
	CsvNode * node;
	long      count = 0;


	if (!list)
		return 0;

	if (!list->head)
		return 0;

	node = list->head;

	while (node)
	{
		if (node->d.data)
			count++;

		node = node->next;
	}

	return count;
}



/* Destroy the CsvData in each node then destroy the
 * node.
 */

int freeCsvDataList(CsvList * list)
{
	CsvNode * node;
	CsvNode * nextNode;


	if (!list)
		return -1;

	node = list->head;

	while (node)
	{
		if (node->d.colSpec)
			freeCsvData(node->d.colSpec);

		nextNode = node->next;

		free(node);

		node = nextNode;
	}

	return 0;
}


int freeCsvRow(CsvRow * row, int cols)
{
	char ** buf;
	int     loop;

	if (!row)
		return -1;

	if ( row->bufs )
	{
		loop = 0;
		buf = row->bufs;

		while (loop < cols)
		{
			if ( *buf )
				free( *buf );

			buf++;
			loop++;
		}

		free(row->bufs);
	}

	free(row);

	return 0;
}


/* Destroy the RowData in each node then destroy the
 * node.  The num value indicates the number of 
 * columns in each row.
 */

int freeCsvRowList(CsvList * list,
				   int       cols)
{
	CsvNode * node;
	CsvNode * nextNode;


	if (!list)
		return -1;

	node = list->head;

	while (node)
	{
		if (node->d.row)
			freeCsvRow(node->d.row, cols);

		nextNode = node->next;

		free(node);

		node = nextNode;
	}

	free(list);

	return 0;
}

/* CsvData contains a dynamically allocated buffer.
 * CsvRow is all dynamically allocated buffers.
 */


/* Here we are whacking the list and some kind of data that
 * goes with it.  For this generic version, it is assumed that
 * only the data item is dynamically allocated and it does
 * not point to anything else that contains dynamically 
 * allocated data
 */

int freeCsvList(CsvList * list)
{
	CsvNode * nextNode;
	CsvNode * node;

	if (!list)
		return -1;

	node = list->head;

	while (node)
	{
		if (node->d.data)
			free(node->d.data);

		nextNode = node->next;

		free(node);

		node = nextNode;
	}

	free(list);

	return 0;
}


/* Nuke each pointer in the array of pointers.
 * Then nuke the array itself.
 */

int freeCsvArray(int     num, 
				 void ** array)
{
	int loop = 0;

	if ( (num < 1) || !array)
		return -1;

	while (loop < num)
	{
		if (*(array + loop))
			free(*(array + loop) );

		loop++;
	}

	free(array);

	return 0;
}


/* Freeing a Control:
 * A Cloned control does not need to free its list or
 * its rows.  IF it is a row then it DOES have its own 
 * rowData (array) to maintain a sort context.
 */

int freeCsvControl(CsvControl * cc, int num)
{
	if (!cc)
		return -1;

	if (cc->list)
	{
		if (cc->type == CSV_CONTROL_ROW)
		{
			if (!cc->cloned)
				freeCsvRowList(cc->list, num);
			else
			{
				if (cc->d.rowData)
					free(cc->d.rowData);
			}
		}
		else if (cc->type == CSV_CONTROL_COLUMN)
		{
			if (!cc->cloned)
				freeCsvDataList(cc->list);
		}
		else
		{
			if (!cc->cloned)
				freeCsvList(cc->list);
		}
	}

	/* Since the list freeing exercise nuked all the data
	 * then the colSpecs/rows/array only needs to be freed,
	 * because all the things its elements point to are 
	 * already gone.  If this is cloned, then skip the
	 * free -- for row controls the rowData is free'd already.
	 */

	if (cc->d.array && !cc->cloned)
		free(cc->d.array);

	free(cc);

	return 0;
}


int freeCsvFile(CsvFile * cf)
{
	if (!cf)
		return -1;

	/* The Rows depend on the number of Columns */

	if (cf->row)
	{
		if (cf->col)
			freeCsvControl(cf->row, cf->col->num);
		else
			freeCsvControl(cf->row, 0);
	}

	if (cf->col)
		freeCsvControl(cf->col, 0);

	return 0;
}

int csvFreeFile(CsvFile * csvFile)
{
	return freeCsvFile(csvFile);
}



CsvNode * createCsvNode(void * data)
{
	CsvNode * newNode;

	if (!data)
		return NULL;

	newNode = (CsvNode *)malloc(sizeof(CsvNode));

	if (newNode)
	{
		newNode->next = NULL;
		newNode->d.data = data;
	}

	return newNode;
}



CsvList * createCsvList(void)
{
	CsvList * cl;

	cl = (CsvList *)malloc(sizeof(CsvList));

	if (!cl)
		return NULL;

	cl->head = NULL;
	cl->tail = NULL;

	return cl;
}


/* Add the data to the end of this list */

int addToCsvList(CsvList * list, void * data)
{
	CsvNode * newNode;

	if (!list || !data)
		return -1;

	newNode = createCsvNode(data);

	if (!newNode)
		return -1;

	if (!list->tail)
		list->head = newNode;
	else
		list->tail->next = newNode;

	list->tail = newNode;

	return 0;
}



static int clearCsvControl(CsvControl * cc)
{
	if (!cc)
		return -1;

	cc->num     = 0L;
	cc->list    = NULL;
	cc->d.array = NULL;
	cc->cloned  = false;

	return 0;
}


/* Note that actual data (Rows or column specs) are
 * created as nodes are added to the list.
 * When the last item is added to the list, then the
 * array of direct pointers to the data is created.
 */

CsvControl * createCsvControl(CsvControlType type)
{
	CsvControl * cc;

	cc = (CsvControl *)malloc(sizeof(CsvControl));

	if (!cc)
		return NULL;

	memset(cc, 0, sizeof(CsvControl));

	cc->type   = type;

	if (clearCsvControl(cc))
	{
		free(cc);
		return NULL;
	}

	cc->list = createCsvList();

	if (!cc->list)
	{
		free(cc);
		return NULL;
	}

	return cc;
}


static CsvFile * createCsvFile(void)
{
	CsvFile * cf;

	cf = (CsvFile *)malloc(sizeof(CsvFile));

	if (!cf)
		return NULL;

	memset(cf, 0, sizeof(CsvFile));

	cf->col = createCsvControl(CSV_CONTROL_COLUMN);

	if (!cf->col)
	{
		free(cf);
		return NULL;
	}

	cf->row = createCsvControl(CSV_CONTROL_ROW);

	if (!cf->row)
	{
		free(cf->col);
		free(cf);
		return NULL;
	}

	return cf;
}


CsvControl * csvFileToColumnControl(char * filename)
{
	CsvControl * cc = NULL;

	if (!filename || !filename)
		return NULL;

	cc = createCsvControl(CSV_CONTROL_COLUMN);

	if (!cc)
		return NULL;

	return cc;
}


/* We now know how many columns there are, so keep looping
 * to add a new CsvData to the list and save a pointer to
 * each new CsvData on the colSpecs array.
 * In the case of CsvData for columns, no structures have
 * been created yet, so we expect that the list and the 
 * array of CsvData pointer must be empty when we get 
 * here.
 */

int createColSpecs(int          count,
				   CsvControl * cc)
{
	int       loop = 0;
	CsvData * newData;

	if ((count < 1) ||     /* useless count */
		!cc ||             /* invalid control */
		(cc->type != CSV_CONTROL_COLUMN) || /* wrong type */
		cc->d.colSpecs ||    /* specs already created!? */
		!cc->list ||       /* list was never started? */
		cc->list->head )   /* list is already populated? */
		return -1;

	cc->d.colSpecs = (CsvData **)malloc(sizeof(CsvData *) * (count + 1) );

	if (!cc->d.colSpecs)
		return -1;

	memset(cc->d.colSpecs, 0, sizeof(CsvData *) * (count + 1) );

	while (loop < count)
	{
		newData = createCsvData();

		if (!newData) /* failed! */
		{
			freeCsvDataList(cc->list);	/* destroy list/columnSpecs if it exists */
			cc->list = NULL;

			free(cc->d.colSpecs);			/* destroy the columnSpec */
			cc->d.colSpecs = NULL;

			return -1;
		}

		if (addToCsvList(cc->list, newData))
		{
			freeCsvDataList(cc->list);
			cc->list = NULL;

			free(cc->d.colSpecs);
			cc->d.colSpecs = NULL;

			return -1;
		}

		*(cc->d.colSpecs + loop) = newData;
		
		loop++;
	}

	cc->num = count;

	return 0;
}



/* Rows have been read from a file, and added to a list.
 * So, here we create a sortable linear array of these 
 * row buffers.
 * First count the working items in the list.
 * Then travel the list, copying the pointer from each 
 * into the flat array.
 * In the case of CsvRow for rows, all rows have
 * been created, so we expect that the list must have
 * nodes on it and the array of CsvRow pointers must be 
 * empty when we get here.
 */

int createRowData(CsvControl * cc)
{
	CsvNode *  node;
	CsvRow  ** row;

	if (!cc ||				/* invalid control */
		!cc->list ||		/* no list created? */
		!cc->list->head ||	/* no nodes on the list? */ 
		(cc->type != CSV_CONTROL_ROW) || /* invalid control type */
		cc->d.rowData)		/* the RowData array is already created? */
		return -1;

	cc->num = countCsvList(cc->list);

	if (cc->num < 1)
		return -1;

	cc->d.rowData = (CsvRow **)malloc(sizeof(CsvRow *) * cc->num );

	if (!cc->d.rowData)
		return -1;

	memset(cc->d.rowData, 0, sizeof(CsvRow *) * cc->num );

	node = cc->list->head;

	row = cc->d.rowData;

	while (node)
	{
		if (node->d.row)
		{
			*row = node->d.row;

			row++;
		}

		node = node->next;
	}

	return 0;
}


/* Mini version of the column parser that does not extract text.
 * Only count the columns 
 */

int countColumns(char * data, 
				 char   sv)
{
	char thisChar;
	bool quotedFlag;
	bool firstChar;
	int  countSep = 1; /* Any data means at least one column */

	if (!data || !*data || !sv)
		return 0;

	quotedFlag = false;
	firstChar  = true;

	while ( (thisChar = *data) ) /* loop while there is data and everything's OK */
	{
		if ((firstChar) && (thisChar == '\"'))
		{
			quotedFlag = true;
			firstChar = false;
			data++;
			continue;
		}

		if ((quotedFlag) && (thisChar == '\"'))
		{
			quotedFlag = false;
			data++;
			continue;
		}

		if (((thisChar == sv) && (!quotedFlag)) ||
			(thisChar == CSV_LF) || /* End of line chars are same */
			(thisChar == CSV_CR))   /* as end of string to parse  */
		{
			firstChar = true;
			countSep++;

			if ((thisChar == CSV_LF) || /* String should end here */
				(thisChar == CSV_CR))
				*data = '\0';
			else
				data++;

			continue;
		}

		data++;
		thisChar = *data;
		firstChar = false;
	}

	return countSep;
}


CsvFile * csvLoadFile(char * filename,
					  char   separator)
{
	FILE    * fp = NULL;
	CsvFile * cf = NULL;
	CsvRow  * cr = NULL;
	char    * inString;
	int       retVal;
	int       columns = 0;

	if (!filename || !*filename || !separator)
		return NULL;

	cf = createCsvFile();

	if (!cf ||
		!cf->col ||
		!cf->col->list ||
		!cf->row ||
		!cf->row->list )
		return NULL;

	fp = fopen(filename, "r");

	if (!fp)
	{
		freeCsvFile(cf);
		return NULL;
	}

	inString = readAndUnEOL(fp);

	while ( inString && !columns )
	{
		columns = countColumns(inString, separator);

		if (!columns)
			inString = readAndUnEOL(fp);
	}

	if (!inString)
	{
		freeCsvFile(cf);
		fclose(fp);
		return NULL;
	}

	/* Here, go create the CsvData structures used for parsing.
	 */

	if (createColSpecs(columns, cf->col))
	{
		freeCsvFile(cf);
		fclose(fp);
		return NULL;
	}

	if (!cf->col->d.colSpecs || !*cf->col->d.colSpecs)
	{
		freeCsvFile(cf);
		fclose(fp);
		return NULL;
	}

	rewind(fp);

	/* Here call the reader to grab a line and then parse it into
	 * the csvData structures.
	 *
	 * Then Create a row, and add it to the list
	 */

	inString = ""; /* just to get into the loop */

	while (inString)
	{
		inString =	readAndUnEOL(fp);

		if (!inString) /* Reached the end of the file! */
			continue;

		if (! *inString)
			continue;

		retVal = csvParse(inString,
						  cf->col->d.colSpecs,
						  separator);
	
		if (retVal != CSV_OK)
			continue;

		cr = createCsvRow(cf->col->d.colSpecs, cf->col->num);
		
		if (!cr)
			continue;
		
		if (addToCsvList(cf->row->list, cr))
		{
			freeCsvRow(cr, cf->col->num);
			continue;
		}
	}

	fclose(fp);

	/* We've read all rows.  Now create our sortable array of Rows */

	if ( createRowData(cf->row) )
	{
		freeCsvFile(cf);
		return NULL;
	}

	return cf;
}


/* Dup the csvFile, clone the column data, dup the row Array */

CsvFile * csvCloneFile(CsvFile * cf)
{
	CsvFile * nf = NULL;

	if (!cf || !cf->col || !cf->row || !cf->row->d.rowData || (cf->row->num < 1) )
		return NULL;

	nf = (CsvFile *)malloc(sizeof(CsvFile));

	if (!nf)
		return NULL;

	memcpy(nf, cf, sizeof(CsvFile));

	nf->col = (CsvControl *)malloc(sizeof(CsvControl));

	if (!nf->col)
	{
		free(nf);
		return NULL;
	}
	memcpy(nf->col, cf->col, sizeof(CsvControl));
	nf->col->cloned = true;

	nf->row = (CsvControl *)malloc(sizeof(CsvControl));

	if (!nf->row)
	{
		free(nf->col);
		free(nf);
		return NULL;
	}
	memcpy(nf->row, cf->row, sizeof(CsvControl));
	nf->row->cloned = true;

	nf->row->d.rowData = (CsvRow **)malloc(sizeof(CsvRow *) * nf->row->num );

	if (!nf->row->d.rowData)
	{
		free(nf->row);
		free(nf->col);
		free(nf);
		return NULL;
	}

	memcpy(nf->row->d.rowData,
		   cf->row->d.rowData,
		   sizeof(CsvRow *) * nf->row->num );

	return nf;
}


long csvGetNumberOfColumns(CsvFile * csvFile)
{
	if (!csvFile)
		return 0;

	if (!csvFile->col)
		return 0;

	if (!csvFile->col->d.colSpecs || !csvFile->col->list)
		return 0;

	if (csvFile->col->num < 0)
		return 0;

	return csvFile->col->num;
}


long csvGetNumberOfRows(CsvFile * csvFile)
{
	if (!csvFile)
		return 0;

	if (!csvFile->row)
		return 0;

	if (!csvFile->row->d.rowData || !csvFile->row->list)
		return 0;

	if (csvFile->row->num < 0)
		return 0;

	return csvFile->row->num;
}


CsvData * csvGetColumnSpec(CsvFile * csvFile, 
						   int       column)
{
	if (!csvFile || 
		!csvFile->col || 
		!csvFile->col->d.colSpecs ||
		(column > csvFile->col->num) ||
		(column < 1) )
		return NULL;

	return csvFile->col->d.colSpecs[column - 1];
}


CsvRow * csvGetRow(CsvFile * csvFile,
				   long      row)
{
	if (!csvFile || 
		!csvFile->row || 
		!csvFile->row->d.rowData ||
		(row > csvFile->row->num) ||
		(row < 1) )
		return NULL;

	return csvFile->row->d.rowData[row - 1];
}


int csvSortOnColumn(CsvFile * csvFile,
					int       col1,  /* column 1 to n.*/
					int       col2,  /* column 1 to n.  -1 is N/A */
					int       col3)  /* column 1 to n.  -1 is N/A */
{
	CsvRow * cr1 = NULL;
	CsvRow * cr2 = NULL;

	int      out;
	int      in;
	int      cmp;


	if (!csvFile || 
		!csvFile->col ||
		(csvFile->col->num < 1) )
		return -1;

	if (csvFile->row->num == 1) return 0;

	if ((col1 < 1) || (col1 > csvFile->col->num) ) return -1;
	if ((col2 < 1) || (col2 > csvFile->col->num) ) col2 = -1;
	if ((col3 < 1) || (col3 > csvFile->col->num) ) col3 = -1;

	/* Bubble up the sort order combinations to the top 
	 * just in case someone is clumsy.
	 */

	if (col1 == col3)
		col3 = -1;

	if (col1 == col2)
	{
		col2 = col3;
		col3 = -1;
	}

	if ((col2 < 1) && (col3 > 0))
	{
		col2 = col3;
		col3 = -1;
	}

	if (col1 > 0) col1--;
	if (col2 > 0) col2--;
	if (col3 > 0) col3--;

	/* The gruesome, brute force sort */

	for (out = 0;
		 out <= (csvFile->row->num - 2);
		 out++)
	{
		for (in = out + 1;
			in <= (csvFile->row->num - 1);
			in++)
		{
			cr1 = csvFile->row->d.rowData[out];
			cr2 = csvFile->row->d.rowData[in];

			if (!cr1)
				continue;
			if (!cr1->bufs)
				continue;

			if (!cr2 || !cr2->bufs) /* and cr1 is valid */
			{
				csvFile->row->d.rowData[out] = cr2;
				csvFile->row->d.rowData[in]  = cr1;
				continue;
			}

			if (cr1->bufs[col1] && cr2->bufs[col1])
			{
				cmp = strcmp(cr1->bufs[col1], cr2->bufs[col1]);

				if (cmp < 0)
					continue;

				if (cmp == 0)
				{
					if (col2 >= 0) 
					{
						if (cr1->bufs[col2] && cr2->bufs[col2])
						{
							cmp = strcmp(cr1->bufs[col2], cr2->bufs[col2]);

							if (cmp < 0)
								continue;

							if (cmp == 0)
							{
								if (col3 >= 0)
								{
									if (cr1->bufs[col3] && cr2->bufs[col3])
									{
										cmp = strcmp(cr1->bufs[col3], cr2->bufs[col3]);

										if (cmp <= 0)
											continue;

										csvFile->row->d.rowData[out] = cr2;
										csvFile->row->d.rowData[in]  = cr1;
										continue;
									}
								}
								else
								{
									if (cr1->bufs[col3]) /* second is less than first */
									{
										csvFile->row->d.rowData[out] = cr2;
										csvFile->row->d.rowData[in]  = cr1;
										continue;
									}
								}
							}
							else if (cmp > 0)
							{
								csvFile->row->d.rowData[out] = cr2;
								csvFile->row->d.rowData[in]  = cr1;
								continue;
							}
						}
						else
						{
							if (cr1->bufs[col2]) /* second is less than first */
							{
								csvFile->row->d.rowData[out] = cr2;
								csvFile->row->d.rowData[in]  = cr1;
								continue;
							}
						}
					}
				}
				else
				{
					csvFile->row->d.rowData[out] = cr2;
					csvFile->row->d.rowData[in]  = cr1;
					continue;
				}
			}
			else /* cr1[col1] or cr2[col1] is NULL */
			{
				if (cr1->bufs[col1]) /* second is less than first */
				{
					csvFile->row->d.rowData[out] = cr2;
					csvFile->row->d.rowData[in]  = cr1;
					continue;
				}
			}
		}
	}

	return 0;
}



CsvRow * csvSearchRow(CsvFile * cf,
					  char    * val1, /* Value to match */
					  int       col1, /* Column to match it in */
					  char    * val2, /* Value to match. NULL is N/A */
					  int       col2, /* Column to match it in. -1 is N/A */
					  char    * val3, /* Value to match. NULL is N/A */
					  int       col3) /* Column to match it in. -1 is N/A */
{
	CsvRow ** cr;
	CsvRow *  crm;
	char   ** crb;

	long low  = 0;
	long high = 0;
	long mid  = 0;
	int  match;


	if (!cf || 
		!cf->col ||
		(cf->col->num < 1) ||
		!cf->row ||
		!cf->row->d.rowData ||
		(cf->row->num < 1) )
		return NULL;

	if (!val1 || 
		(col1 < 1) || (col1 > cf->col->num) ) return NULL;
	if ((col2 < 1) || (col2 > cf->col->num) ) col2 = -1;
	if ((col3 < 1) || (col3 > cf->col->num) ) col3 = -1;

	if (col1 == col3)
		col3 = -1;

	if (col1 == col2)
	{
		col2 = col3;
		val2 = val3;
		col3 = -1;
	}

	if ((col2 < 1) && (col3 > 0))
	{
		col2 = col3;
		val2 = val3;
		col3 = -1;
	}

	if (col1 > 0) col1--;
	if (col2 > 0) col2--;
	if (col3 > 0) col3--;

	if (col2 == -1) val2 = NULL;
	if (col3 == -1) val3 = NULL;

	cr = cf->row->d.rowData;

	high = cf->row->num - 1;

	while (low <= high)
	{
		mid = (low + high) / 2;

		crm = *(cr + mid);   /* Our pointer in the rowData */

		if (crm)
			crb = crm->bufs; /* Our bufs from the pointer */
		else
			crb = NULL;

		if (crm && crb && *(crb + col1) )
		{
			match = strcmp(*(crb + col1), val1);

			if (!match)
			{
				if (col2 > 0)
				{
					if ( *(crb + col2) )
					{
						match = strcmp(*(crb + col2), val2);

						if (!match)
						{
							if (col3 > 0) 
							{
								if ( *(crb + col3) )
									match = strcmp(*(crb + col3), val3);
								else
									match = -1;
							}
						}
					}
					else
						match = -1;
				}
			}
		}
		else
			match = -1;

		if (!match)
			return crm;

		if (match > 0)
			high = mid - 1;
		else
			low = mid + 1;
	}

	return NULL;
}

