/***************************************************************************
 *   $Id: csvFile.h,v 1.3 2005/02/27 04:26:53 kenjen Exp $ *
 *                                                                         *
 *   Created: 2002/09/30 17:26:18                                          *
 *                                                                         *
 *   Copyright (C) 2002-2005 by Kenneth Jennings                           *
 *   ken_jennings@bellsouth.net                                            *
 *                                                                         *
 ***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CSV_FILE_STRUCT_INCL
#define CSV_FILE_STRUCT_INCL

#include <csvData.h>

typedef enum
{
	CSV_CONTROL_COLUMN = 1,
	CSV_CONTROL_ROW
} CsvControlType;


typedef struct _csv_row
{
	char ** bufs;
} CsvRow;


typedef struct _csv_node
{
	struct _csv_node * next;
	union
	{
		void         * data;
		CsvData      * colSpec;
		CsvRow       * row;
	} d;
} CsvNode;


typedef struct _csv_list
{
	CsvNode * head;
	CsvNode * tail;
} CsvList;


typedef struct _csv_control
{
	long           num;   /* Number of things read/detected in file */
	CsvList      * list;  /* List built as things are read */
	CsvControlType type;  /* Row or Column? */
	bool           cloned;/* Flag indicates the things pointed to by 
						   * this control are clones of another struct */
	union 
	{
		void    ** array;    /* final array of column specs/sortable rows */
		CsvData ** colSpecs; /* final array of column specs */
		CsvRow  ** rowData;  /* final array of sortable rows */
	} d;
} CsvControl; 


typedef struct _csv_file
{
	CsvControl * col; /* Column management */
	CsvControl * row; /* Row management */
} CsvFile;



CsvFile * csvLoadFile(char * filename,
					  char   separator);

long csvGetNumberOfColumns(CsvFile * csvFile);

long csvGetNumberOfRows(CsvFile * csvFile);

CsvData * csvGetColumnSpec(CsvFile * csvFile, 
						   int       column);

CsvRow * csvGetRow(CsvFile * csvFile,
				   long      row);

int csvSortOnColumn(CsvFile * csvFile,
					int       col1,  /* column 1 to n.*/
					int       col2,  /* column 1 to n.  -1 is N/A */
					int       col3); /* column 1 to n.  -1 is N/A */

CsvRow * csvSearchRow(CsvFile * csvFile,
					  char    * val1, /* Value to match */
					  int       col1, /* Column to match it in */
					  char    * val2, /* Value to match. NULL is N/A */
					  int       col2, /* Column to match it in. -1 is N/A */
					  char    * val3, /* Value to match. NULL is N/A */
					  int       col3);/* Column to match it in. -1 is N/A */

int csvFreeFile(CsvFile * csvFile); /* Disassemble and nuke everything */

CsvFile * csvCloneFile(CsvFile * csvFile); /* Dup the csvFile, clone the column data, dup the row Array */

#endif

