/***************************************************************************
 *   $Id: main.c,v 1.7 2007/02/20 10:11:40 kenjen Exp $ *
 *                                                                         *
 *   Created: Wed Jun  5 11:06:21 EDT 2002                                 *
 *                                                                         *
 *   Copyright (C) 2002-2005 by Kenneth Jennings                           *
 *   ken_jennings@bellsouth.net                                            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include <unistd.h>

#include "checking2html.h"

static char rcsid[] = "$Id: main.c,v 1.7 2007/02/20 10:11:40 kenjen Exp $";

extern char * getRCSIDchecking2html( void );
extern char * getRCSIDmiscUtils( void );
extern char * getRCSIDcsvData( void );
extern char * getRCSIDcsvFile( void );

/* Command line parameter settings */

static char * ifile = "checking.csv";     /* -i  */
static char * ofile = NULL;               /* -o  */
static char * tfile = NULL;               /* -t  */
static char * ltext = NULL;               /* -l  */
static char * col1  = "AAAAFF";           /* -c1 */
static char * col2  = "8888DD";           /* -c2 */
static char * col3  = "AAAAFF";           /* -c3 */
static char * col4  = "DDDDDD";           /* -c4 */
static char * col5  = "AAFFAA";           /* -c5 */

/* list of files we will open */

static FILE * inputFile       = NULL;
/* static FILE * outputFile      = NULL;     */
static FILE * templateFile    = NULL;


char * getHColor(HColor n)
{
	static char * osfa = "AAAAAA"; /* One Size Fits All for errors */

	switch (n)
	{
		case HC_TITLE_ROW:  return col1; break;
		case HC_ODD_MONTH:  return col2; break;
		case HC_EVEN_MONTH: return col3; break;
		case HC_ODD_ROW:    return col4; break;
		case HC_EVEN_ROW:   return col5; break;
	}

	return osfa;
}



/* Original Comment:
 * I would much rather use something like getopt in unix, but since
 * DOS doesn't support it, I have to do everything the hard way.
 * Revised Comment:
 * I'm sick of DOS/Windows, and since I'm moving myself entirely
 * to linux, why should I accommodate that Winslop useless
 * bag of programmer puke? So, we're using getopt again which
 * works fine in unix and Amiga.
 */

int doParms(int     argc,
			char ** argv)
{
	int    c;
	int    ncolor;
	char * color;

	if (argc == 1) /* 1 means only the program name is in argv */
		return 1;

	while ( ( c = getopt(argc, argv, "i:o:t:l:c:h") ) != EOF)
	{
		switch (c)
		{
			case 'i':
				ifile = optarg;
				break;

			case 'o':
				ofile = optarg;
				break;

			case 't':
				tfile = optarg;
				break;

			case 'l':
				ltext = optarg;
				break;

			case 'c': /* -c 1=AA88FF */
				if (!optarg) /* No paramter string given? */
					break;
				ncolor = atoi(optarg);
				if ( (ncolor < HC_MIN) || (ncolor > HC_MAX) ) /* No usable number given? */
					break;
				color = strchr(optarg, '='); 
				if (!color) /* No '=' separator? */
					break;
				color++;
				if (!*color) /* No string following the '=' */
					break;
				if (strlen(color) != 6) /* Color string too short? */
					break;
				switch (ncolor)
				{
					case HC_TITLE_ROW:  col1 = color; break; /* Title Row */
					case HC_ODD_MONTH:  col2 = color; break; /* Jan, Mar, May ... */
					case HC_EVEN_MONTH: col3 = color; break; /* Feb, Apr, Jun ... */
					case HC_ODD_ROW:    col4 = color; break; /* Odd Rows 1, 3, 5... */
					case HC_EVEN_ROW:   col5 = color; break; /* Even Rows 2, 4, 6... */
				}
				break;

			case 'h':
			default:
				return 1;
				break;
		} /* switch(c) */
	} /* while ( getopt() ) */

	return 0;
}



void usage(void)
{
	time_t      t  = 0;
	struct tm * tt = NULL;
	int         y  = 0;

	/* Some silliness to determine this year in order to 
	 * update the "perpetual" copyright.
	 */
	if ( time(&t) != ((time_t)-1) )
	{
		tt = localtime(&t);
		if (tt)
		{
			y = tt->tm_year + 1900;
			if ( y <= 2002 )
				y = 0;
		}
	}

	fprintf(stderr,
			"\nUsage: checking2html [-i checking.csv] \n"
			"                     [-t template.html]\n"
			"                     [-o output.html]\n"
			"                     [-c 1=color string]\n"
			"                     [-c 2=color string]\n"
			"                     [-c 3=color string]\n"
			"                     [-c 4=color string]\n"
			"                     [-c 5=color string]\n"
			"                      -l \"Label Text\"\n\n"
			"\t-i    Checking account CSV file name.  Default is \"checking.csv\"\n\n"
			"\t-t    Template HTML file name. HTML table substituted for <CHECKING> tag.\n\n"
			"\t-o    Specify output file name.  Default is STDOUT.\n\n"
			"\t-c 1= Background hex color of cells in Table Title row. Default AAAAFF (lt blue)\n\n"
			"\t-c 2= Background hex color of Jan, Mar, etc. cells. Default 8888DD (dk blue)\n\n"
			"\t-c 3= Background hex color of Feb, Apr, etc. cells. Default AAAAFF (lt blue)\n\n"
			"\t-c 4= Background hex color of row 1, 3, 5, etc. Default DDDDDD (lt grey)\n\n"
			"\t-c 5= Background hex color of row 2, 4, 6, etc. Default AAFFAA (lt green)\n\n" 
			"\t-l    Text for the title at the top of the table.\n\n"
			"\t-h    Help.  Display this usage information.\n\n"
			"Another fine product from the Republican Club of Davie/Cooper City.\n"
			"www.DavieGOP.org (c) 2002" );
	if ( y )
		fprintf(stderr, "-%d", y );

	fprintf(stderr, "\n\n%s\n%s\n%s\n%s\n%s\n\n",
			rcsid,
			getRCSIDchecking2html(),
			getRCSIDmiscUtils(),
			getRCSIDcsvData(),
			getRCSIDcsvFile() );
	fflush(NULL);
}



int main(int    argc,
		 char * argv[])
{
	FILE * outputFile = NULL;

	/* Parse the command line parameters */

	if (doParms(argc,
				argv))
	{
		usage();
		exit(-1);
	}

	/* Try the input files */

	inputFile = fopen(ifile, "r");
	if (!inputFile)
	{
		fprintf(stderr,
				"FAILED to open the file \"%s\".\n\n",
				ifile);
		exit(-1);
	}
	LoadChecking(inputFile);
	fclose(inputFile);

	/* Figure out where the output is going to go to */

	if (ofile)
	{
		outputFile = fopen(ofile,
						   "w");
		if (!outputFile)
		{
			fprintf(stderr,
					"FAILED to open the output file \"%s\".\n\n",
					ofile);
			exit(-1);
		}
	}
	else
		outputFile = stdout;

	/* Do we output the template first? */

	if (tfile)
	{
		templateFile = fopen(tfile, "r");

		if (!templateFile)
		{
			fprintf(stderr,
					"FAILED to open the template file \"%s\".  Skipping Template.\n\n",
					tfile);
			tfile = NULL;
		}
	}

	if (tfile)
		dumpUntilTag(templateFile,
					 outputFile,
					 "<CHECKING>");

	/* Output table */

	if (!buildHTML(outputFile,
				   gimmeCheckingList(),
				   ltext))
	{
		fprintf(stderr,
				"Failed to build checking account table.\n");
		exit(-1);
	}

	if (tfile)
	{
		dumpUntilEnd(templateFile,
					 outputFile);

		fclose(templateFile);
	}

	if (ofile)
		fclose(outputFile);

	return EXIT_SUCCESS;
}

