/***************************************************************************
 *   $Id: miscUtils.c,v 1.5 2007/02/20 10:11:40 kenjen Exp $
 *                                                                         *
 *   Created: Mon Sep 30 2002
 *                                                                         *
 *   Copyright (C) 2002-2005 by Kenneth Jennings                           *
 *   ken_jennings@bellsouth.net                                            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "csvType.h" /* We're using bool */
#include "miscUtils.h"

static char rcsid[] = "$Id: miscUtils.c,v 1.5 2007/02/20 10:11:40 kenjen Exp $";

char * getRCSIDmiscUtils( void ) { return rcsid; }


/* Checking list.
 * I add transactions to the list at the end.
 * The assumption is that the file is already sorted by date.
 */

/*
char * strdup(char * s)
{
	int    n;
	char * c;

	if (!s)
		return NULL;

	n = strlen(s) + 1;

	c = (char *)malloc(n);

	if (c)
		strcpy(c, s);

	return c;
}
*/

/* return <0 is s<t, 0 if s==t, >0 if s>t */
/*
int strcasecmp(char * s,
			   char * t)
{
	char sc = '\0';
	char tc = '\0';

	sc = toupper(*s);
	tc = toupper(*t);

	while (sc == tc)
	{
		if (sc == '\0')
			return 0;

		s++;
		t++;
		sc = toupper(*s);
		tc = toupper(*t);
	}

	return sc - tc;
}
*/

void strtolower(char * s)
{
	if (!s || !*s)
		return;

	while (*s)
	{
		*s = tolower(*s );
		s++;
	}
}

void strtoupper(char * s)
{
	if (!s || !*s)
		return;

	while (*s)
	{
		*s = toupper( *s );
		s++;
	}
}

/* Uppercase the first character of each word,
 * lowercase everything after.
 */

void strtomixed(char * s)
{
	bool wasSpaceOrMinus = true;

	if (!s || !*s)
		return;

	while (*s)
	{
		if ( isalpha(*s) )
		{
			if (wasSpaceOrMinus)
				*s = toupper(*s);
			else
				*s = tolower(*s);

			wasSpaceOrMinus = false;
		}

		if ( isspace(*s) || (*s == '-') )
			wasSpaceOrMinus = true;

		s++;
	}
}


/* Remove all spaces in a string. */

void strtodeblank(char * s)
{
	char * d = s;

	if (!s || !*s)
		return;

	while (*s)
	{
		if (isspace(*s))
		{
			s++;
			continue;
		}

		*d++ = *s++;
	}

	*d = '\0';

	return;
}

/* Convert int to string, add "&nbsp;" to right align */

char * inttonbspstring(int val,
					   int len)
{
	static char buf[80];
	int         width;

	sprintf(buf, "%d", val);

	width = strlen(buf);

	if (width >= len)
		return buf;

	width = len - width; /* How many nbsp ? */

	buf[0] = '\0';

	while (width)
	{
		len = strlen(buf);
		sprintf(buf + len,
				"&nbsp;");
		width--;
	}

	len = strlen(buf);
	sprintf(buf + len, "%d", val);

	return buf;
}

/* convert string to right aligned with nbsp */

char * strtonbspstring(char * s,
					   int    len)
{
	static char buf[80];
	int         width;

	buf[0] = '\0';

	if (!s || !*s)
		return buf;

	width = strlen(s);

	if (width >= len)
		return s;

	width = len - width; /* How many nbsp ? */

	buf[0] = '\0';

	while (width)
	{
		len = strlen(buf);
		sprintf(buf + len,
				"&nbsp;");
		width--;
	}

	strcat(buf, s);

	return buf;
}


/* Make a percent string right aligned to 3.2% digits (7 chars) */

char * strpercentstring(int val,
						int denom)
{
	static char buf[9];
	double result;

	if (denom > 0)
	{
		result = (double)val * 100.0 / (double)denom;
		result = (double)((long)(result * 100.0 + .5)); /* round to 2 digits */
		result = result / 100; /* put two digits back after decimal point */
	}
	else
		result = 0;

	sprintf(buf, "%.2f%%", result);

	return (strtonbspstring(buf, 7));
}


/* If empty or NULL, return the &nbsp; string, otherwise return the parameter */

char * nbspOrStr(char * s)
{
	static char * nbsp = "&nbsp;";

	if (!s || !*s)
		return nbsp;

	return s;
}


/* Decide if the string needs to be generated as html HREF or if
 * plain text is fine. if it starts http: or mailto: it is href'd
 *
 * Simple hack version: if there is a colon it is href'd
 */

char * hrefortext(char * s)
{
	static char buf[250];
	char * colon;

	buf[0] = '\0';

	if (!s || !*s)
		return buf;

	colon = strchr(s, ':');

	if (!colon)
		return s;

	colon++; /* Skip to what follows the colon */

	sprintf(buf,
			"<A HREF=\"%s\">%s</A>",
			s,
			colon);

	return buf;
}




/* Read Bytes from source file, search byte stream for tag,
 * Return when tag is found.  Note that all bytes prior to
 * the tag will be written to the output, but not the tag
 * itself.  Note that this is a very stupid function and
 * any appropriate set of chars could be matched if it is
 * quoted or whatever.  Make sure the tag can be identified
 * when it is intended.
 */

int dumpUntilTag(FILE * infile,
				 FILE * outfile,
				 char * tag)
{
	char * buf;
	int    blen  = 0;
	int    tlen;
	bool   found = false;
	int    newchar;
	int    loop;


	if (!infile || !outfile || !tag || !*tag)
		return -1;

	/* See, make buffer to match the size of the tag string */

	buf = strdup(tag);

	if (!buf)
		return -1;

	tlen = strlen(tag);

	while (!found && ( ( newchar = fgetc(infile)) != EOF) )
	{
		*(buf + blen) = (char)newchar;

		if (toupper(*(buf + blen)) == toupper(*(tag + blen)) )
		{
			blen++;
			if (blen == tlen)
			{
				found = true;
				continue;
			}
		}
		else
		{
			loop = 0;

			while (loop <= blen)
			{
				if (fputc((int) *(buf + loop), outfile) == EOF)
					return -1;

				loop++;
			}

			blen = 0;
		}
	}

	if (!found)
		return -1;

	return 0;
}




/* Quick-N-Dirty -- Just dump bytes until we've copied the
 * rest of the input file to the output file.
 */

int dumpUntilEnd(FILE * infile,
				 FILE * outfile)
{
	int    newchar;


	if (!infile || !outfile)
		return -1;

	while (( ( newchar = fgetc(infile)) != EOF) )
	{
		if (fputc((int) newchar, outfile) == EOF)
			return -1;
	}

	return 0;
}


/* moneyStr()
 * extract the sign, digits, decimal point from
 * a string. (Parenthesis count as - sign.)
 * Reconvert to a text string as money value
 * with parenthesis as necessary for negative value.
 * This allows the input to be an OOo 1.0 CSV
 * which will be represented as a signed value
 * using decimal points if needed or as a MS Works
 * CSV file which could be parenthesis as sign
 * with money symbol and commas.
 * Note: code references to commas and money
 * symbols are specific to US dollars.  This
 * could someday stand some additional work to
 * internationalize the function.
 * Note this builds a string with no real checks
 * on lengths, so this is potentially very baaad.
 * However, the CSV routines supplying data limit
 * money strings to 11 characters which should
 * keep us safe from buffer overruns.
 */
int moneyStr(char * in,
			 char * out)
{
	float  bucks;
	bool   isNeg = false;
	char   temp[30];
	int    cnt;
	char * i;
	char * o;

	if (!in  || !*in ||
		!out)
		return -1;

	*out = '\0';

	i = in;
	o = temp;

	while(*i)
	{
		if (isdigit(*i) ||
			(*i == '.') )
		{
			*o = *i;
			o++;
		}

		if ( (*i == '(') ||
			 (*i == '-') )
			isNeg = true;

		i++;
	}

	*o = '\0';

	/* There apparently is no usable money string,
	 * so return value is still successful
	 */
	if (!temp[0])
		return 0;

	bucks = atof(temp);
	sprintf(temp, "%#.2f", bucks); /* clean decimal */

	/* An easy and clever way to put the
	 * thousands separators in is needed
	 * for this. Or maybe even just a stupid
	 * brute force one like this below.
	 */

	i = temp;
	o = out;

	cnt = strlen(temp);

	if (isNeg)
		*o++ = '(';

	*o++ = '$';

	while (*i)
	{
		*o++ = *i++;

		if ((cnt > 6) && ( (cnt % 3) == 1) )
			*o++ = ',';

		cnt--;
	}

	if (isNeg)
		*o++ = ')';

	*o = '\0';

	return 0;
}
